import xarray as xr
from pylab import *
from numpy import *
#from glob import glob
#from os import path
import time
import dask
from dask.diagnostics import ProgressBar
import shutil
import pickle

#this is a minimal code that illustrates issue with .to_zarr() or .to_netcdf when writing a dataset with datetime64 data

#outputDir is the name of the zarr output; it should be set to a location on a fast filesystem with enough space
outputDir='./testOut.zarr'

def testToZarr(dimensions,haveTimeType=True):
    '''This code writes out an empty dataset with the dimensions specified
    in the "dimensions" arguement, and returns the time it took to
    create the dask delayed object and the time it took to compute the
    delayed object.

    if haveTimeType is True, the "time" variable has type datetime64
    '''

    if haveTimeType:
        #specify the type of variables to be written out. Each has dimensions (trajectory,obs)
        varType={'time':dtype('datetime64[ns]'),
                 'age':dtype('float32'),
                 'lat':dtype('float32'),
                 'lon':dtype('float32'),
                 'z':dtype('float32'),
                 }
    else:
        varType={'time':dtype('float32'),
                 'age':dtype('float32'),
                 'lat':dtype('float32'),
                 'lon':dtype('float32'),
                 'z':dtype('float32'),
                 }
        

    #now make an empty dataset 
    dataOut=xr.Dataset()

    #now add the empty variables
    for v in varType:
        vEmpty=dask.array.zeros((dimensions['trajectory'],dimensions['obs']),dtype=varType[v])
        dataOut=dataOut.assign({v:(('trajectory','obs'),vEmpty)})

    #chunk data
    chunksize={'trajectory':5*int(1e4),'obs':10}
    print('   chunking dataset to',chunksize)
    dataOut=dataOut.chunk(chunksize)

    #create dask delayed object, and time how long it took
    tic=time.time()
    if True: #write to zarr
        delayedObj=dataOut.to_zarr(outputDir,compute=False)
    else: #write to netCDF
        delayedObj=dataOut.to_netcdf(outputDir,compute=False)
    createGraphTime=time.time()-tic
    print('   created graph in',createGraphTime)

    #execute the delayed object, and see how long it took. Use progress bar
    tic=time.time()
    with ProgressBar():
        results=delayedObj.compute()
    writeOutTime=time.time()-tic
    print('   wrote data in',writeOutTime)

    return createGraphTime,writeOutTime,dataOut


#now lets do some benchmarking
if __name__ == "__main__":
    figure(1,figsize=(10.0,8.0))
    clf()
    style.use('ggplot')

    #make a vector that is the size of the first dimension
    #("trajectory") of the data set
    trajectoryVec=logspace(log10(1000),log10(1.4e8),20).astype(int)

    #pre-allocate variables to store results
    createGraphTimeVec_time=0.0*trajectoryVec+nan
    writeOutTimeVec_time=0.0*trajectoryVec+nan

    createGraphTimeVec_noTime=0.0*trajectoryVec+nan
    writeOutTimeVec_noTime=0.0*trajectoryVec+nan

    #get data for various array sizes
    for n in range(len(trajectoryVec)):

        #write out data, and benchmark time
        dimensions={'trajectory':trajectoryVec[n],'obs':245}
        print('starting to write file of dimensions',dimensions,'for case with time variable')

        shutil.rmtree(outputDir,ignore_errors=True)
        createGraphTimeVec_time[n],writeOutTimeVec_time[n],dataOut_time=testToZarr(dimensions,haveTimeType=True)

        shutil.rmtree(outputDir,ignore_errors=True)
        createGraphTimeVec_noTime[n],writeOutTimeVec_noTime[n],dataOut_noTime=testToZarr(dimensions,haveTimeType=False)

        print('   done')

        #now plot output
        clf()

        subplot(2,1,1)
        loglog(trajectoryVec,createGraphTimeVec_time,'r-o',label='with datetime64 variable')
        loglog(trajectoryVec,createGraphTimeVec_noTime,'b-*',label='without datetime64 variable')
        #xlabel('size of first dimensions')
        ylabel('seconds')
        title('time to create dask graph',fontsize='medium')
        legend()
        firstAx=axis()

        subplot(2,1,2)
        loglog(trajectoryVec,writeOutTimeVec_time,'r-o',label='with datetime64 variable')
        loglog(trajectoryVec,writeOutTimeVec_noTime,'b-*',label='without datetime64 variable')
        axis(xmin=firstAx[0],xmax=firstAx[1])
        xlabel('size of first dimensions')
        ylabel('seconds')
        title('time to write data to disk',fontsize='medium')
        legend()

        draw()
        show()
        pause(0.01)

        #save the figure each time, since this code can crash as the
        #size of the dataset gets larger and the dataset with
        #datetime64[ns] causes .to_zarr() to exhaust all the memory
        savefig('jnk_out.png',dpi=100)


